#include "proximity.h"

proximity *allocprox(sparse* mat) {
	proximity *prox=malloc(sizeof(proximity));

	prox->n=mat->dim;
	prox->scores=malloc(mat->dim*sizeof(idval));
	prox->ns=0;

	return prox;
}

unsigned* allocdeg(sparse* mat){
	unsigned *deg=malloc(mat->dim*sizeof(unsigned));
	unsigned i;
	for (i=0;i<mat->dim;i++) {
		deg[i]=mat->degcum[i+1]-mat->degcum[i];
	}
	return deg;
}

fifo* allocfifo(unsigned n) {
	fifo *q=malloc(sizeof(fifo));
	q->s=n;
	q->tab=malloc(n*sizeof(unsigned));
	q->intab=calloc(n,sizeof(bool));
    q->in = 0;
	q->out = 0;
	return q;
}

void put(fifo *q,unsigned new) {
	if (q->intab[new]==0) {
    	q->tab[q->in] = new;
    	q->in = (q->in + 1) % q->s;
		q->intab[new]=1;
	}
}

char notempty(fifo *q) {
	if (q->in == q->out)
		return 0; /* Queue Empty - nothing to get*/
	return 1;
}

unsigned get(fifo *q) {
    unsigned old;
	old = q->tab[q->out];
    q->out = (q->out + 1) % q->s;
	q->intab[old]=0;
    return old;
}

static int compare (void const *a, void const *b) {
   return ( (*(idval*)a).val - (*(idval*)b).val > 0 )?-1:1;
}

void computeprox(sparse* mat,proximity* prox,unsigned node){
	unsigned i,j,k,n=mat->dim;
	double tmp;

	static fifo* que;
	static unsigned* deg;
	static double* vec;
	static double* ppr;
	static unsigned* list;//nodes with nonzero ppr or vec
	static bool *inlist;//inlist[i]=1 iff i is in list
	static bool b=1;
	if (b) {
		que=allocfifo(n);
		deg=allocdeg(mat);
		vec=calloc(n,sizeof(double));
		ppr=calloc(n,sizeof(double));
		list=malloc(n*sizeof(unsigned));//nodes with nonzero ppr or vec
		inlist=calloc(n,sizeof(bool));//inlist[i]=1 iff i is in list
		b=0;
	}
	unsigned nlist=0;//number of nodes in list

	prox->node=node;
	vec[node]=1.;
	inlist[node]=1;
	list[nlist++]=node;
	put(que,node);
	while (notempty(que)){
		i=get(que);
		//push function
		ppr[i]+=ALPHA*vec[i];
		tmp=vec[i]*(1.-ALPHA)/deg[i];
		vec[i]=0;
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
			k=mat->neigh[j];
			vec[k]+=tmp;
			if (inlist[k]==0){
				inlist[k]=1;
				list[nlist++]=k;
			}
			if (vec[k]>=EPS*deg[k]){
				put(que,k);
			}
		}
		//end of push function
	}
	//forming scores
	prox->ns=0;
	for (i=0;i<nlist;i++) {
		j=list[i];
		if (ppr[j]>0){
			prox->scores[prox->ns].val=ppr[j]/deg[j];
			prox->scores[prox->ns++].id=j;
			ppr[j]=0;
		}
		vec[j]=0;
		inlist[j]=0;
	}
	qsort(prox->scores,prox->ns, sizeof(idval), compare);
}


void mkcom(proximity* prox){
	unsigned n=prox->ns;
	double mi=MI,ma=(log10(n-1)<MA)?log10(n-1):MA;
	unsigned i,size=0;
	double p,pm;

	static double *x;
	static double *y;
	static unsigned *z;
	static bool b=1;
	if (b) {
		x=malloc(NX*sizeof(double));
		y=malloc(NX*sizeof(double));
		z=malloc(NX*sizeof(unsigned));
		b=0;
	}

	for (i=0;i<NX;i++){
		z[i]=floor(pow(10,(mi+(ma-mi)*i/(NX-1.))));
		y[i]=log10(prox->scores[z[i]].val);
		x[i]=log10(z[i]);
	}

	for (i=0;i<NX-2*NY;i++){
		if (z[i+2*NY]==z[i]) break;
		p=(y[i+2*NY]-y[i])/(x[i+2*NY]-x[i]);
		if (p<pm){
			size=i;
			pm=p;
		}
	}

	prox->size=z[size+NY];

	prox->slope=-pm;
}

void printcom(proximity* prox){
	char str[50]="test/",num[20];
	unsigned i;
	sprintf(num, "%u", prox->node);
	strcat(str, num);
	strcat(str, "_");
	sprintf(num, "%u", prox->size);
	strcat(str, num);
	strcat(str, "_");
	sprintf(num, "%le", prox->slope);
	strcat(str, num);
	FILE* file=fopen(str,"w");
	for (i=0;i<prox->ns;i++) {
		fprintf(file,"%u %le\n",prox->scores[i].id,prox->scores[i].val);
	}
}




